  ' program geosync1.bas     July 31, 2017
  
  ' equilibrium longitudes, geocentric radii
  ' and longitude acceleration of satellites
  ' in geosynchronous orbits
  
  ''''''''''''''''''''''''''
  
  option default float
  
  option base 1
  
  dim c(3, 3), s(3, 3)
  
  dim xas, c22, c31, c33, g1, g2 
  
  ' utility constants
  
  const pi2 = 2.0 * pi
  
  ' conversion factor - degrees to radians
  
  const dtr = pi / 180.0
  
  ' conversion factor - radians to degrees
  
  const rtd = 180.0 / pi
  
  ' define egm96 astrodynamic constants
  
  const mu = 398600.4415
  
  const req = 6378.1363
  
  const xj20 = 1.08262668355e-003
  
  const xj40 = -1.61962159137e-006
  
  const omega = 7.292115e-5
  
  ' unnormalized egm96 gravity model c and s coefficients
  
  c(2, 2) = 1.57446037456e-006
  s(2, 2) = -9.03803806639e-007
  
  c(3, 1) = 2.19263852917e-006
  s(3, 1) = 2.68424890297e-007
  
  c(3, 3) = 1.00548778064e-007
  s(3, 3) = 1.97222559006e-007
  
  ' compute gravity coefficients
  
  const xj22 = sqr(c(2, 2)^2 + s(2, 2)^2)
  
  const xj31 = sqr(c(3, 1)^2 + s(3, 1)^2)
  
  const xj33 = sqr(c(3, 3)^2 + s(3, 3)^2)
  
  ' compute gravity model longitudes
  
  const xlambda22 = (1.0 / 2.0) * atn(s(2, 2) / c(2, 2))
  
  const xlambda31 = (1.0 / 1.0) * atn(s(3, 1) / c(3, 1))
  
  const xlambda33 = (1.0 / 3.0) * atn(s(3, 3) / c(3, 3))
  
  print " "
  print "program geosync1"
  print " "
  print "longitudes and radii of geosynchronous satellites"
  print "-------------------------------------------------"
  print " "
  
  ' set root-finding tolerance
  
  rtol = 1.0e-10
  
  ' find equilibrium location #1
  ' ----------------------------
  
  x1 = 70.0 * dtr
  
  x2 = 80.0 * dtr
  
  root(x1, x2, rtol, xroot, froot)
  
  elong1 = xroot
  
  ' compute cnm coefficients
  
  rr = req / xas
  
  c22 = xj22 * rr^2
  
  c31 = xj31 * rr^3
  
  c33 = xj33 * rr^3
  
  as1 = xas
  
  ' longitude acceleration
  
  geofunc3(elong1, xldd1)
  
  ' drift cycle period
  
  tau1 = pi / sqr(-3.0 * g2)
  
  print " "
  print "equilibrium location #1"
  print "-----------------------"
  print " "
  print "east longitude       ", str$(rtd * elong1, 0, 4) + " degrees"
  print " "
  print "radius               ", str$(as1, 0, 4) + " kilometers"
  print " "
  print "drift acceleration   ", str$(rtd * xldd1, 0, -10) + " degrees/day^2"
  print " "
  print "drift cycle period   ", str$(tau1, 0, 4) + " days"
  print " "
  
  ' find equilibrium location #2
  ' ----------------------------
  
  x1 = 160.0 * dtr
  
  x2 = 170.0 * dtr
  
  root(x1, x2, rtol, xroot, froot)
  
  elong2 = xroot
  
  ' compute cnm coefficients
  
  rr = req / xas
  
  c22 = xj22 * rr^2
  
  c31 = xj31 * rr^3
  
  c33 = xj33 * rr^3
  
  as2 = xas
  
  ' longitude acceleration
  
  geofunc3(elong2, xldd2)
  
  print " "
  print "equilibrium location #2"
  print "-----------------------"
  print " "
  print "east longitude       ", str$(rtd * elong2, 0, 4) + " degrees"
  print " "
  print "radius               ", str$(as2, 0, 4) + " kilometers"
  print " "
  print "drift acceleration   ", str$(rtd * xldd2, 0, -10) + " degrees/day^2"
  print " "
  
  ' find equilibrium location #3
  ' ----------------------------
  
  x1 = 250.0 * dtr
  
  x2 = 260.0 * dtr
  
  root(x1, x2, rtol, xroot, froot)
  
  elong3 = xroot
  
  ' compute cnm coefficients
  
  rr = req / xas
  
  c22 = xj22 * rr^2
  
  c31 = xj31 * rr^3
  
  c33 = xj33 * rr^3
  
  as3 = xas
  
  ' longitude acceleration
  
  geofunc3(elong3, xldd3)
  
  ' drift cycle period
  
  tau3 = pi / sqr(-3.0 * g2)
  
  print " "
  print "equilibrium location #3"
  print "-----------------------"
  print " "
  print "east longitude       ", str$(rtd * elong3, 0, 4) + " degrees"
  print " "
  print "radius               ", str$(as3, 0, 4) + " kilometers"
  print " "
  print "drift acceleration   ", str$(rtd * xldd3, 0, -10) + " degrees/day^2"
  print " "
  print "drift cycle period   ", str$(tau3, 0, 4) + " days"
  print " "
  
  ' find equilibrium location #4
  ' ----------------------------
  
  x1 = 340.0 * dtr
  
  x2 = 350.0 * dtr
  
  root(x1, x2, rtol, xroot, froot)
  
  elong4 = xroot
  
  ' compute cnm coefficients
  
  rr = req / xas
  
  c22 = xj22 * rr^2
  
  c31 = xj31 * rr^3
  
  c33 = xj33 * rr^3
  
  as4 = xas
  
  ' longitude acceleration
  
  geofunc3(elong4, xldd4)
  
  print " "
  print "equilibrium location #4"
  print "-----------------------"
  print " "
  print "east longitude       ", str$(rtd * elong4, 0, 4) + " degrees"
  print " "
  print "radius               ", str$(as4, 0, 4) + " kilometers"
  print " "
  print "drift acceleration   ", str$(rtd * xldd4, 0, -10) + " degrees/day^2"
  print " "
  
end
  
  '''''''''''''''''
  '''''''''''''''''
  
sub geofunc1(x, fx)
  
  ' equilibrium longitude objective function
  
  ''''''''''''''''''''''''''''''''''''''''''
  
  local xlambdas, ask, rr, da
  
  ' load current value of east longitude
  
  xlambdas = x
  
  ' compute geosynchronous satellite radius
  
  ask = (mu / omega^2)^(1.0 / 3.0)
  
  rr = req / ask
  
  da = 2.0 * xj20 * ask * rr^2 + 12.0 * xj22 * ask * rr^2 * cos(2.0 * (xlambdas - xlambda22))
  
  da = da - 8.0 * xj31 * ask * rr^3 * cos(3.0 * (xlambdas - xlambda31))
  
  da = da + 80.0 * xj33 * ask * rr^3 * cos(3.0 * (xlambdas - xlambda33))
  
  xas = ask + da
  
  ' compute radius ratio
  
  rr = req / xas
  
  ' evaluate "g" functions
  
  g1 = 6.0 * xj22 * rr^2 * sin(2.0 * (xlambdas - xlambda22))
  
  g1 = g1 - 1.5 * xj31 * rr^3 * sin(xlambdas - xlambda31)
  
  g1 = g1 + 45.0 * xj33 * rr^3 * sin(3.0 * (xlambdas - xlambda33))
  
  g2 = 12.0 * xj22 * rr^2 * cos(2.0 * (xlambdas - xlambda22))
  
  g2 = g2 - 1.5 * xj31 * rr^3 * cos(xlambdas - xlambda31)
  
  g2 = g2 + 135.0 * xj33 * rr^3 * cos(3.0 * (xlambdas - xlambda33))
  
  ' compute objective function
  
  fx = g1 / g2
  
end sub
  
  '''''''''''''''''''''''''
  '''''''''''''''''''''''''
  
sub geofunc3(xlambda, xldd)
  
  ' longitude-drift acceleration
  
  ''''''''''''''''''''''''''''''
  
  local tmp1, omegae
  
  tmp1 = 18.0 * (c22 * sin(2.0 * (xlambda - xlambda22)) - 0.25 * c31 * sin(xlambda - xlambda31) + 7.5 * c33 * sin(3.0 * (xlambda - xlambda33)))
  
  omegae = omega * 86400.0
  
  ' longitude acceleration (rad/day^2)
  
  xldd = tmp1 * omegae * omegae
  
end sub

  '''''''''''''''''''''''''''''''''
  '''''''''''''''''''''''''''''''''
  
sub root(x1, x2, tol, xroot, froot)
  
  ' real root of a single non-linear function subroutine
  
  ' input
  
  '  x1  = lower bound of search interval
  '  x2  = upper bound of search interval
  '  tol = convergence criter%ia
  
  ' output
  
  '  xroot = real root of f(x) = 0
  '  froot = function value
  
  ' note: requires sub geofunc1
  
  '''''''''''''''''''''''''''''
  
  local eps, a, b, c, d, e, fa, fb, fcc, tol1
  
  local xm, p, q, r, s, xmin, tmp
  
  eps = 2.23e-16
  
  e = 0.0
  
  a = x1
  
  b = x2
  
  geofunc1(a, fa)
  
  geofunc1(b, fb)
  
  fcc = fb
  
  for iter% = 1 to 50
    
    if (fb * fcc > 0.0) then
      
      c = a
      
      fcc = fa
      
      d = b - a
      
      e = d
      
    end if
    
    if (abs(fcc) < abs(fb)) then
      
      a = b
      
      b = c
      
      c = a
      
      fa = fb
      
      fb = fcc
      
      fcc = fa
      
    end if
    
    tol1 = 2.0 * eps * abs(b) + 0.5 * tol
    
    xm = 0.5 * (c - b)
    
    if (abs(xm) <= tol1 or fb = 0.0) then exit for
    
    if (abs(e) >= tol1 and abs(fa) > abs(fb)) then
      
      s = fb / fa
      
      if (a = c) then
        
        p = 2.0 * xm * s
        
        q = 1.0 - s
        
      else
        
        q = fa / fcc
        
        r = fb / fcc
        
        p = s * (2.0 * xm * q * (q - r) - (b - a) * (r - 1.0))
        
        q = (q - 1.0) * (r - 1.0) * (s - 1.0)
        
      end if
      
      if (p > 0.0) then q = -q
      
      p = abs(p)
      
      min = abs(e * q)
      
      tmp = 3.0 * xm * q - abs(tol1 * q)
      
      if (min < tmp) then min = tmp
      
      if (2.0 * p < min) then
        
        e = d
        
        d = p / q
        
      else
        
        d = xm
        
        e = d
        
      end if
      
    else
      
      d = xm
      
      e = d
      
    end if
    
    a = b
    
    fa = fb
    
    if (abs(d) > tol1) then
      
      b = b + d
      
    else
      
      b = b + sgn(xm) * tol1
      
    end if
    
    geofunc1(b, fb)
    
  next iter%
  
  froot = fb
  
  xroot = b
  
end sub
